import type { PagicPlugin } from '../Pagic.ts';
// eslint-disable-next-line no-duplicate-imports
import type Pagic from '../Pagic.ts';
import { path } from '../../deps.ts';

export type PagicConfigSidebar = Record<string, OnePagicConfigSidebar>;

export type OnePagicConfigSidebar = (
  | {
      text?: string;
      link?: string;
      children?: OnePagicConfigSidebar;
      expanded?: boolean;
    }
  | string
)[];

export type PagePropsSidebar = {
  text: string;
  link?: string;
  pagePath?: string;
  expanded?: boolean;
  children?: PagePropsSidebar;
}[];

const sidebar: PagicPlugin = {
  name: 'sidebar',
  insert: 'after:tsx',
  fn: async (pagic) => {
    if (!pagic.config.sidebar) {
      return;
    }

    for (const pagePath of pagic.pagePaths) {
      const pageProps = pagic.pagePropsMap[pagePath];

      let parsedSidebar: Record<string, PagePropsSidebar> = {};
      for (const [prefix, oneConfig] of Object.entries({ ...pagic.getConfig(pagePath).sidebar })) {
        parsedSidebar[prefix] = parseSidebarConfig(oneConfig, pagic);
      }

      for (const [prefix, pagePropsSidebar] of Object.entries(parsedSidebar)) {
        if (`/${pageProps.outputPath}`.startsWith(prefix)) {
          pagic.pagePropsMap[pagePath] = {
            ...pageProps,
            sidebar: pagePropsSidebar,
          };
          break;
        }
      }
    }
  },
};

function parseSidebarConfig(sidebarConfig: OnePagicConfigSidebar, pagic: Pagic): PagePropsSidebar {
  return sidebarConfig
    .map((sidebarConfigItem) => {
      if (typeof sidebarConfigItem === 'string') {
        const sidebarConfigGlob = path.globToRegExp(sidebarConfigItem);
        return Object.entries(pagic.pagePropsMap)
          .filter(([key]) => sidebarConfigGlob.test(key))
          .map(([key, value]) => ({
            text: value.title,
            link: value.outputPath,
            pagePath: value.pagePath,
          }));
      }
      // Deep clone
      let item = JSON.parse(JSON.stringify(sidebarConfigItem)) as PagePropsSidebar[0];

      if (typeof item.link !== 'undefined') {
        if (typeof pagic.pagePropsMap[item.link] === 'undefined') {
          item.text = item.text ?? item.link;
        } else {
          item.pagePath = item.link;
          item.text = item.text ?? pagic.pagePropsMap[item.link].title ?? item.link;
          item.link = pagic.pagePropsMap[item.link].outputPath;
        }
      }
      if (Array.isArray(item.children)) {
        item.children = parseSidebarConfig(item.children, pagic);
      }
      return item;
    })
    .flat();
}

export default sidebar;
